/* -----------------------------------------------------------------------
 * Project:	ut-snmpv2 project
 * Filename:	snmpInitAgentSys.c
 * Date:	April 1994
 * Purpose:
 * Author:	Michel van Ooijen & Edward Veluwenkamp
 * Remarks:
 * -----------------------------------------------------------------------
 */

/* #include <snmpInitAgent.h>
 */
#include <snmpInitPerkins.h>
#include <snmpInitAgentSys.h>


/* -----------------------------------------------------------------------
 *  Function:   getipadrname
 *  Arguments:  ipaddr, string which contains IPADDRESS in base 256 notation (d.d.d.d) (OUT)
 *              name, string (IN)
 *  Purpose:    Gets IPAddress of the system with hostname name
 *  Remarks:
 * -----------------------------------------------------------------------
*/
int SINgetipadrname(ipaddr,name)
   char *ipaddr,*name; 
   {
	struct 	hostent	*he ;

       	he = gethostbyname(name);                  /* get IP-address from hostname */
        if (he != NULL) 
      	  {if (he != (struct hostent *)0) 
             strcpy( ipaddr, inet_ntoa( *((struct in_addr *) (he->h_addr))) );
            return(0);
          }
        else return(-1);
    }




/* -----------------------------------------------------------------------
 *  Function:   getipaddress
 *  Arguments:  IPAddr, string which contains IPADDRESS in base 256 notation (d.d.d.d) (OUT)
 *  Purpose:    Gets IPAddress of the system 
 *  Remarks:
 * -----------------------------------------------------------------------
*/
void SINgetipaddress(ipaddr)
   char *ipaddr; 
   {
	struct 	hostent	*he ;
        char    name[ARLEN];
    
        if (gethostname(name,MAXSTRLEN) == 0)
	  {
	   he = gethostbyname(name);                      /* get IP-address from hostname */
           if (he != (struct hostent *)0) 
           strcpy( ipaddr, inet_ntoa( *((struct in_addr *) (he->h_addr))) );
          }
        else
          {
           fprintf(stdout,"\t\tERROR: unable to find hostname.\n");
           name[0]='\0';
           ipaddr[0]='\0';
          }
   }


/* -----------------------------------------------------------------------
 *  Function:   checkkey
 *  Arguments:  testKey, string which contains a 32 hex digit key (OUT)
 *  Purpose:    Test if key (first 16 digits) don't contain weak/semi weak keys for DES algortihm
 *  Remarks:    last 16 digits are not tested (initialisation vector for DES)
 * -----------------------------------------------------------------------
*/
int SINcheckkey(testkey)
  char *testkey;
  {  
   int i;

   char checkArray[16][16] ={"0101010101010101","fefefefefefefefe","1f1f1f1f1f1f1f1f","e0e0e0e0f1f1f1f1","01fe01fe01fe01fe",
                             "1fe01fe01fe01fe0","01e001e001f101f1","1ffe1ffe010e010e","011f011f010e010e","e0fee0fef1fef1fe",
                             "fe01fe01fe01fe01","e01fe01ff10ef10e","e001e001f101f101","fe1ffe1ffe0efe0e","1f011f010e010e01",
                             "fee0fee0fef1fef1"};

   for (i=0; i<16;i++)
           if (strncmp(testkey,checkArray[i],16) == 0)
                 return(-1);
   return (0);
  }




/* -----------------------------------------------------------------------
 *  Function:   getrandomkey
 *  Arguments:  RandomKey, string which contains a 32 hex digit key (OUT)
 *  Purpose:    Generate a randomKey of 32 hex digits
 *  Remarks:    
 * -----------------------------------------------------------------------
*/
void SINgetrandomkey(randomkey)
  char *randomkey;
  {
   struct timeval tp; 
   char   s1[ARLEN];
   short  s2;
   int    i,seed;

   do
    {     
     if (gettimeofday(&tp,NULL) != -1)       /* fills in a entry in the struct timeb with system-time */ 
       seed = (int) tp.tv_sec + tp.tv_usec; 
   /*  printf("%u ",(int)seed);*/
     srand(seed);                /* Generate a new sequence of pseudo-random number to be returned */
                                 /* by subsequent calls to the function rand()                     */
     for (i=0; i < 8; i++)       
       {                        /* fill s1 with key generated by rand() */
        s2 = (short) rand();
        memcpy((void *) &s1[i*2],(const void *) &s2,2);
       }
     sprintf(randomkey,"%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x",(unsigned char) s1[0],(unsigned char) s1[1],(unsigned char) s1[2],
            (unsigned char) s1[3],(unsigned char) s1[4],(unsigned char) s1[5],(unsigned char) s1[6],(unsigned char)       
            (unsigned char) s1[7],(unsigned char) s1[8],(unsigned char) s1[9],(unsigned char) s1[10],(unsigned char) s1[11],
            (unsigned char) s1[12],(unsigned char) s1[13],(unsigned char) s1[14],(unsigned char) s1[15]);
    }
   while (SINcheckkey(randomkey) != 0);          
  }
